import datetime
from itertools import accumulate

import matplotlib.pyplot as plt
import pandas as pd
from strongtyping.strong_typing import match_typing

from sprint_velocity.utils import create_dates, current_date


@match_typing
def generate_plot(json_data: dict, sprint_start_date: str, outputfile, sprint_interval: int = 14):
    data = []
    sprint_start = datetime.datetime.strptime(sprint_start_date, "%Y-%m-%d").date()
    sprint_end = sprint_start + datetime.timedelta(days=sprint_interval)
    dates = list(create_dates(sprint_start, sprint_end))
    try:
        date_today = dates.index(current_date())
    except ValueError:
        date_today = None

    title = f"Sprint from {sprint_start}: Issues"
    total = json_data["total"]
    issues = json_data["issues"]
    for issue in issues:
        created = issue["fields"]["created"]
        updated = issue["fields"]["updated"]
        status = issue["fields"]["status"]["statusCategory"]["key"]
        status_name = issue["fields"]["status"]["name"]
        data.append(
            {
                "created": created,
                "status": status,
                "status_name": status_name,
                "updated": updated,
            }
        )
    df = pd.DataFrame(data)

    df["created"] = pd.to_datetime(df["created"])
    df["updated"] = pd.to_datetime(df["updated"])

    sprint_issues = df[df["created"].dt.date <= sprint_start]
    dummy_ea = sprint_issues.groupby(sprint_issues["created"].dt.date)["created"].count()
    total_sprint_issues = dummy_ea.values.sum()

    df_after_sprint_start = df[df["created"].dt.date >= sprint_start]
    df_after_sprint_start_count = df_after_sprint_start.groupby(
        df_after_sprint_start["created"].dt.date
    )["created"].count()

    df_sprint = df[(df["updated"].dt.date >= sprint_start) & (df["status"] == "done")]
    df_resolved_count = df_sprint.groupby(df_sprint["updated"].dt.date)["status"].count()

    bar_x = list(range(len(dates)))
    bar_height = [
        df_after_sprint_start_count.get(datetime.date.fromordinal(obj.toordinal()), 0)
        for obj in dates
    ]

    resolved_values = [
        df_resolved_count.get(datetime.date.fromordinal(obj.toordinal()), None) for obj in dates
    ]
    resolved_values = [obj for obj in resolved_values if obj is not None]
    resolved_values[0] = total - resolved_values[0]

    open_issues = [
        df_after_sprint_start_count.get(datetime.date.fromordinal(obj.toordinal()), 0)
        for obj in dates
    ]
    open_issues[0] = total_sprint_issues

    x = list(range(len(dates)))[: len(resolved_values)]
    y = list(accumulate(resolved_values, lambda x_, y_: x_ - y_))

    plt.figure()
    plt.title(title)
    plt.step(x=x, y=y, color="r", label="Issues Open")

    plt.bar(bar_x, bar_height, color="silver", label="In Sprint Added")
    plt.ylabel("Issues")

    sprint_days = list(range(len(dates)))  #
    results = [total * (1 - (idx * 0.1)) for idx in sprint_days]

    plt.plot(sprint_days, results, color="navy", label="Base Line")

    if date_today:
        plt.plot(
            [date_today, date_today],
            [total + 5, 0],
            color="black",
            linestyle="dashed",
            label="Current Sprint-Date",
        )

    # plt.xticks(x, dates, rotation=45)
    plt.legend()
    plt.grid(True)
    plt.tight_layout()
    plt.savefig(f"{outputfile}.png", format="png")



import datetime
import shutil
import urllib.parse
from pathlib import Path
from typing import List, Optional

import typer
from requests import request

from sprint_velocity.plot_matplolib import generate_plot
from sprint_velocity.utils import (
    display_settings,
    get_current_sprint_info,
    get_header,
    get_json_data,
    get_project,
    save_json_data,
)

app = typer.Typer()


@app.command()
def settings_backupfile(backup_file_path: Path, date_format: str):
    json_data = get_json_data()
    json_data["backup_file_path"] = backup_file_path.as_posix()
    json_data["date_format"] = date_format
    save_json_data(json_data)


@app.command()
def settings_filestorage(file_path: Path):
    json_data = get_json_data()
    json_data["file_path"] = file_path.as_posix()
    save_json_data(json_data)


@app.command()
def settings_board(
    company: str,
    board_id: int,
    token: str,
    url: str = typer.Option(..., help="https://jira.foo.de"),
):
    json_data = get_json_data()
    json_data[company] = {board_id: {"token": token, "url": f"{url}/rest"}}
    save_json_data(json_data)


@app.command()
def display_config():
    json_data = get_json_data()
    display_settings(json_data)


@app.command()
def velocity_graph(company: str, board_ids: Optional[List[int]] = typer.Argument(None)):
    json_data = get_json_data()
    if not (company_data := json_data.get(company)):
        typer.secho(
            f"Please create a setting for {company} via settings_board.",
            fg=typer.colors.RED,
        )
        raise typer.Abort

    ids = board_ids if board_ids else []

    for board_id in company_data:
        if ids and int(board_id) not in ids:
            continue

        token = company_data[board_id]["token"]
        url = company_data[board_id]["url"]

        headers = get_header(token)
        project = get_project(url, board_id, headers)
        sprint_info = get_current_sprint_info(url, board_id, headers)
        sprint_start_date = sprint_info["start_date"]

        search_str = f"project = {project} and issuetype in subTaskIssueTypes() AND Sprint = {sprint_info['id']} AND (resolution = unresolved or resolved >= {sprint_start_date})"  # noqa: E501
        search_query = urllib.parse.quote(search_str)

        base_url = f"{url}/api/2/search?jql={search_query}"
        response = request("GET", url=base_url, headers=headers)

        file_root = Path(json_data.get("file_path", ""), project)
        outputfile = file_root / Path(sprint_info["name"])
        if not file_root.exists():
            file_root.mkdir(parents=True)

        if backup_file_path := json_data.get("backup_file_path"):
            new_file = Path(f"{outputfile}.png")
            if new_file.exists():
                filename = outputfile.split("/")[-1]
                date_str = datetime.date.today().strftime(json_data.get("date_format", "%Y-%m-%d"))
                backup_file = Path(backup_file_path) / Path(f"{filename}_{date_str}.png")
                shutil.copy(new_file, backup_file)

        generate_plot(response.json(), sprint_start_date, outputfile)

        typer.echo(f"Plot was saved under {outputfile}.png")

    typer.echo("All plots processed.")


if __name__ == "__main__":
    app()



import datetime
import json
from json import JSONDecodeError
from pathlib import Path

import pendulum
import typer
from requests import request
from requests.structures import CaseInsensitiveDict
from strongtyping.strong_typing import match_typing


@match_typing
def create_dates(start_end: datetime.date, end_date: datetime.date):
    period = pendulum.period(start=start_end, end=end_date)
    for dt in period.range("days"):
        if dt.weekday() not in (5, 6):
            yield dt


def current_date():
    return datetime.date.today()


def get_json_data() -> dict:
    with Path("config.json").open("r") as file:
        try:
            return json.load(file)
        except JSONDecodeError:
            return {}


@match_typing
def save_json_data(json_data: dict):
    with Path("config.json").open("w") as file:
        json.dump(json_data, file)


@match_typing
def get_header(token: str) -> dict:
    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    if not token:
        if token_ := get_json_data().get("token"):
            headers["Authorization"] = f"Bearer {token_}"
        else:
            typer.secho("No Token in settings!!!", fg=typer.colors.RED)
            typer.secho(
                "Please add a token to the settings via [settings_token <token>]",
                fg=typer.colors.GREEN,
            )
            raise typer.Abort
    else:
        headers["Authorization"] = f"Bearer {token}"
    return headers


@match_typing
def get_current_sprint_info(url: str, board_id: int, headers: dict):
    base_url = f"{url}/agile/1.0/board/{board_id}/sprint"
    response = request("GET", url=base_url, headers=headers)

    for obj in response.json()["values"]:
        if obj["state"] == "active":
            return {
                "id": obj["id"],
                "start_date": obj["startDate"].split("T")[0],
                "name": obj["name"].lower().replace(" ", "_"),
            }


@match_typing
def get_project(url: str, board_id: int, headers):
    base_url = f"{url}/agile/1.0/board/{board_id}/project"
    response = request("GET", url=base_url, headers=headers)
    values = response.json()["values"]
    return values[0]["key"]


def display_settings(data: dict, tab_level: int = 0):
    idx = 0
    tab = "\t" * tab_level
    for key, val in data.items():
        color = typer.colors.GREEN if idx % 2 == 0 else typer.colors.CYAN
        if isinstance(val, dict):
            typer.secho(f"{tab}{key}: ", fg=color)
            display_settings(val, tab_level=tab_level + 1)
        else:
            typer.secho(f"{tab}{key}: {val}", fg=color)
        idx += 1
